﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.IO;
using System.Linq;
using System.Net;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Xml;

namespace TestTSXml
{
    public partial class Form1 : Form
    {
        string loginXml = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n" +
"<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n" +
"  <soap:Body>\r\n" +
"    <Login xmlns=\"http://TrackService.TrackRoad.com/\">\r\n" +
"      <userName>string</userName>\r\n" +
"      <password>string</password>\r\n" +
"    </Login>\r\n" +
"  </soap:Body>\r\n" +
"</soap:Envelope>";

        string logoutXml = "<?xml version=\"1.0\" encoding=\"utf-8\"?>\r\n" +
"<soap:Envelope xmlns:xsi=\"http://www.w3.org/2001/XMLSchema-instance\" xmlns:xsd=\"http://www.w3.org/2001/XMLSchema\" xmlns:soap=\"http://schemas.xmlsoap.org/soap/envelope/\">\r\n" +
"  <soap:Header>\r\n" +
"    <SessionIDHeader xmlns=\"http://TrackService.TrackRoad.com/\">\r\n" +
"      <SessionID>string</SessionID>\r\n" +
"    </SessionIDHeader>\r\n" +
"  </soap:Header>\r\n" +
"  <soap:Body>\r\n" +
"    <Logout xmlns=\"http://TrackService.TrackRoad.com/\" />\r\n" +
"  </soap:Body>\r\n" +
"</soap:Envelope>";

        public Form1()
        {
            InitializeComponent();
            cbxHost.SelectedIndex = 0;
            cbxAction.SelectedIndex = 1;
            txtRequest.Text = loginXml;
        }

        private void btnMakeRequest_Click(object sender, EventArgs e)
        {
            btnMakeRequest.Enabled = false;
            try
            {
                txtResponse.Text = "";
                string post = lblPost.Text;
                string host = (string)cbxHost.SelectedItem;
                string soapAction = (string)cbxAction.SelectedItem;
                string url = host + post;
                string content = txtRequest.Text;
                byte[] contentBytes = System.Text.Encoding.UTF8.GetBytes(content);

                HttpWebRequest req = (HttpWebRequest)HttpWebRequest.Create(url);
                req.Method = "POST";
                req.ContentType = "text/xml; charset=utf-8";
                req.ContentLength = contentBytes.Length;
                req.Headers.Add("SOAPAction", soapAction);
                req.Timeout = 1000 * 60 * 180;//3 hours

                Stream sw = req.GetRequestStream();
                sw.Write(contentBytes, 0, contentBytes.Length);
                sw.Close();

                HttpWebResponse resp = (HttpWebResponse)req.GetResponse();

                StreamReader reader = new StreamReader(resp.GetResponseStream());

                string responseXml = reader.ReadToEnd();
                txtResponse.Text = FormatXml(responseXml);

                reader.Close();
                resp.Close();
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
                return;
            }
            finally
            {
                btnMakeRequest.Enabled = true;
            }
        }

        public static String FormatXml(String XML)
        {
            String Result = "";

            MemoryStream mStream = new MemoryStream();
            XmlTextWriter writer = new XmlTextWriter(mStream, Encoding.Unicode);
            XmlDocument document = new XmlDocument();

            try
            {
                // Load the XmlDocument with the XML.
                document.LoadXml(XML);

                writer.Formatting = Formatting.Indented;

                // Write the XML into a formatting XmlTextWriter
                document.WriteContentTo(writer);
                writer.Flush();
                mStream.Flush();

                // Have to rewind the MemoryStream in order to read
                // its contents.
                mStream.Position = 0;

                // Read MemoryStream contents into a StreamReader.
                StreamReader sReader = new StreamReader(mStream);

                // Extract the text from the StreamReader.
                String FormattedXML = sReader.ReadToEnd();

                Result = FormattedXML;
            }
            catch (XmlException)
            {
            }

            mStream.Close();
            writer.Close();

            return Result;
        }

        private void btnOpenFromFile_Click(object sender, EventArgs e)
        {
            if (openFileDialog1.ShowDialog() == System.Windows.Forms.DialogResult.OK)
            {
                using (StreamReader sr = new StreamReader(File.OpenRead(openFileDialog1.FileName)))
                {
                    txtRequest.Text = sr.ReadToEnd();
                }
            }
        }

        private void cbxAction_SelectedIndexChanged(object sender, EventArgs e)
        {
            switch (cbxAction.SelectedIndex)
            {
                case 0:
                    txtRequest.Text = loginXml;
                    break;
                case 6:
                    txtRequest.Text = logoutXml;
                    break;
                default:
                    txtRequest.Text = "";
                    break;
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            txtResponse.Clear();
        }

        private void btnExportExcel_Click(object sender, EventArgs e)
        {
            try
            {
                if (string.IsNullOrWhiteSpace(txtResponse.Text))
                    return;

                if (saveFileDialog1.ShowDialog() != System.Windows.Forms.DialogResult.OK)
                    return;

                XmlParser.XmlParser xmlParser = new XmlParser.XmlParser();
                XmlParser.Types.DispatchResult dispatchResult = xmlParser.ParseDispatchResultXml(txtResponse.Text);
                string csv = ExportCSV(dispatchResult);
                File.WriteAllText(saveFileDialog1.FileName, csv);

            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

        private string ExportCSV(XmlParser.Types.DispatchResult dispatchResult)
        {
            //string header = "\"No\",\"Latitude\",\"Longitude\",\"Name\",\"Street\",\"City\",\"PostalCode\",\"Country\",\"Distance\",\"Time\",\"WaitingTime\",\"Weight\",\"Boxes\",\"Phone\",\"Description\",\"ControlNo\"";
            //stream.WriteLine(header);

            DataTable table = new DataTable();
            table.Columns.Add("No", typeof(string));
            table.Columns.Add("Latitude", typeof(decimal));
            table.Columns.Add("Longitude", typeof(decimal));
            table.Columns.Add("Name", typeof(string));
            table.Columns.Add("Street", typeof(string));
            table.Columns.Add("City", typeof(string));
            table.Columns.Add("PostalCode", typeof(string));
            table.Columns.Add("Country", typeof(string));
            table.Columns.Add("Distance", typeof(double));
            table.Columns.Add("Time", typeof(int));
            table.Columns.Add("WaitingTime", typeof(int));
            table.Columns.Add("Weight", typeof(double));
            table.Columns.Add("WeightSum", typeof(double));
            table.Columns.Add("MaxWeight", typeof(double));
            table.Columns.Add("MaxWeightWarning", typeof(string));

            table.Columns.Add("TimeEstimatedArrival", typeof(DateTime));
            table.Columns.Add("TimeConstraintArrival1", typeof(DateTime));
            table.Columns.Add("TimeConstraintDeparture1", typeof(DateTime));
            table.Columns.Add("TimeConstraintArrival2", typeof(DateTime));
            table.Columns.Add("TimeConstraintDeparture2", typeof(DateTime));
            table.Columns.Add("CanArriveEarly", typeof(bool));


            table.Columns.Add("Boxes", typeof(int));
            table.Columns.Add("Phone", typeof(string));
            table.Columns.Add("Description", typeof(string));
            table.Columns.Add("ControlNo", typeof(string));

            if(dispatchResult.Items!=null)
            {
                var items = dispatchResult.Items.OrderBy(x => x.Vehicle.Name);

                foreach (var item in items)
                {
                    double weightSum = 0.0;
                    //System.Diagnostics.Debug.WriteLine("-------------------------------------");
                    DataRow row1 = table.NewRow();
                    table.Rows.Add(row1);
                    for (int i = 0; i < item.Locations.Count; i++)
                    {
                        XmlParser.Types.Location loc = item.Locations[i];
                        //System.Diagnostics.Debug.WriteLine("{0} {1}", loc.Name, loc.Weight);
                        double lat = 0.0;
                        double lon = 0.0;
                        string street = null, city = null, zip = null, country = null;
                        if (loc.LatLong != null)
                        {
                            lat = loc.LatLong.Latitude;
                            lon = loc.LatLong.Longitude;
                        }

                        if (loc.Address != null)
                        {
                            street = loc.Address.Street;
                            city = loc.Address.City;
                            zip = loc.Address.PostalCode;
                            country = loc.Address.Country;
                        }
                        weightSum += loc.Weight;

                        DataRow row = table.NewRow();
                        row["No"] = i.ToString("0000");
                        row["Latitude"] = lat.ToString("f7");
                        row["Longitude"] = lon.ToString("f7");
                        row["Name"] = loc.Name;
                        row["Street"] = street;
                        row["City"] = city;
                        row["PostalCode"] = zip;
                        row["Country"] = country;
                        row["Distance"] = loc.Distance;
                        row["Time"] = loc.Time;
                        row["WaitingTime"] = loc.Wait;
                        row["Weight"] = loc.Weight;
                        row["WeightSum"] = weightSum;
                        row["MaxWeight"] = item.Vehicle.MaxWeight;
                        row["MaxWeightWarning"] = (weightSum > item.Vehicle.MaxWeight ? "WARNING" : "");
                        row["TimeEstimatedArrival"] = loc.TimeEstimatedArrival;
                        row["TimeConstraintArrival1"] = loc.TimeConstraintArrival;
                        row["TimeConstraintDeparture1"] = loc.TimeConstraintDeparture;
                        row["TimeConstraintArrival2"] = loc.TimeConstraintArrival2;
                        row["TimeConstraintDeparture2"] = loc.TimeConstraintDeparture2;
                        row["CanArriveEarly"] = loc.CanArriveEarly;
                        row["Boxes"] = loc.Skids;
                        row["Boxes"] = loc.Skids;
                        row["Phone"] = loc.Phone;
                        row["Description"] = loc.Description;
                        row["ControlNo"] = "";


                        table.Rows.Add(row);

                    }
                }

            }

            StringBuilder sb = new StringBuilder();

            string[] columnNames = table.Columns.Cast<DataColumn>().Select(column => column.ColumnName).ToArray();
            sb.AppendLine(string.Join(",", columnNames));

            foreach (DataRow row in table.Rows)
            {
                IEnumerable<string> fields = row.ItemArray.Select(field => string.Concat("\"", field.ToString().Replace("\"", "\"\""), "\""));
                sb.AppendLine(string.Join(",", fields));
            }

            return sb.ToString();
        }

        private void btnOpenXmlResponse_Click(object sender, EventArgs e)
        {
            try
            {
                if (openFileDialog1.ShowDialog() != System.Windows.Forms.DialogResult.OK)
                    return;

                txtResponse.Text = File.ReadAllText(openFileDialog1.FileName);
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.ToString());
            }
        }

    }
}
